@@ -4,6 +4,11 @@ Note:
   1. Arrays organized by dictionary, each key is corresponding to a sample component
   2. Each sample component is a sub-dictionary, each key is corresponding to a input feature's array 
 
+
+TODO:
+    Numpy input format has been replaced by pandas DataFrame.
+    Should remove numpy_io or add support for different input formats?
+
 """
 import logging
 import pathlib
@@ -104,7 +109,9 @@ def get_cut_pass_index_dict(
                     if feature_array is None:
                         feature_array = temp_array
                     else:
-                        feature_array = np.concatenate((feature_array, temp_array))
+                        feature_array = np.concatenate(
+                            (feature_array, temp_array)
+                        )
 
                 feature_array = feature_array
 
@@ -130,7 +137,9 @@ def get_cut_pass_index_dict(
                     pass_index = temp_pass_index
                 else:
                     pass_index = np.intersect1d(pass_index, temp_pass_index)
-            logger.debug(f"Got cut-passed index for {array_type} {sample_component}")
+            logger.debug(
+                f"Got cut-passed index for {array_type} {sample_component}"
+            )
             if pass_index is not None:
                 logger.debug(f"Shape {pass_index.shape}")
             pass_id_dict[array_type][sample_component] = pass_index
@@ -264,7 +273,9 @@ def load_npy_arrays(
             # use float32 to reduce memory consumption, risky?
             feature_array = np.array(feature_array, dtype=np.float32)
             if pass_index is not None:
-                sample_array_dict[feature] = feature_array[pass_index].flatten()
+                sample_array_dict[feature] = feature_array[
+                    pass_index
+                ].flatten()
             else:
                 sample_array_dict[feature] = feature_array.flatten()
         if "weight" in out_features:
