@@ -9,19 +9,24 @@ import numpy as np
 import pandas as pd
 from matplotlib.ticker import NullFormatter
 
+import hepynet.common.hepy_type as ht
 from hepynet.common import common_utils
 from hepynet.data_io import array_utils
-from hepynet.evaluate import evaluate_utils
-from hepynet.train import hep_model
 
 logger = logging.getLogger("hepynet")
 
 
-def calculate_asimov(sig, bkg):
+def calculate_asimov(sig: float, bkg: float):
     return math.sqrt(2 * ((sig + bkg) * math.log(1 + sig / bkg) - sig))
 
 
-def calculate_significance(sig, bkg, sig_total=None, bkg_total=None, algo="asimov"):
+def calculate_significance(
+    sig: float,
+    bkg: float,
+    sig_total: float = None,
+    bkg_total: float = None,
+    algo="asimov",
+):
     """Returns asimov significance"""
     # check input
     if sig <= 0 or bkg <= 0 or sig_total <= 0 or bkg_total <= 0:
@@ -48,24 +53,29 @@ def calculate_significance(sig, bkg, sig_total=None, bkg_total=None, algo="asimo
     elif algo == "s_sqrt_sb":
         return sig / math.sqrt(sig + bkg)
     elif algo == "asimov_rel":
-        return calculate_asimov(sig, bkg) / calculate_asimov(sig_total, bkg_total)
+        return calculate_asimov(sig, bkg) / calculate_asimov(
+            sig_total, bkg_total
+        )
     elif algo == "s_b_rel":
         return (sig / sig_total) / (bkg / bkg_total)
     elif algo == "s_sqrt_b_rel":
         return (sig / sig_total) / math.sqrt(bkg / bkg_total)
     elif algo == "s_sqrt_sb_rel":
-        return (sig / sig_total) / math.sqrt((bkg + sig) / (sig_total + bkg_total))
+        return (sig / sig_total) / math.sqrt(
+            (bkg + sig) / (sig_total + bkg_total)
+        )
     else:
-        logger.warning("Unrecognized significance algorithm, will use default 'asimov'")
+        logger.warning(
+            "Unrecognized significance algorithm, will use default 'asimov'"
+        )
         return calculate_asimov(sig, bkg)
 
 
 def get_significances(
-    model_wrapper: hep_model.Model_Base,
     df: pd.DataFrame,
-    job_config,
-    significance_algo="asimov",
-    multi_class_cut_branch=0,
+    job_config: ht.config,
+    significance_algo: str = "asimov",
+    multi_class_cut_branch: int = 0,
 ):
     """Gets significances scan arrays.
 
@@ -81,18 +91,12 @@ def get_significances(
     ic = job_config.input.clone()
     # prepare signal
     sig_df = array_utils.extract_sig_df(df)
-    sig_predictions, _, _ = evaluate_utils.k_folds_predict(
-        model_wrapper.get_model(), sig_df[ic.selected_features].values
-    )
-    if sig_predictions.ndim == 2:
-        sig_predictions = sig_predictions[:, multi_class_cut_branch]
+    sig_predictions = sig_df[["y_pred"]].values
+    sig_predictions = sig_predictions[:, multi_class_cut_branch]
     # prepare background
     bkg_df = array_utils.extract_bkg_df(df)
-    bkg_predictions, _, _ = evaluate_utils.k_folds_predict(
-        model_wrapper.get_model(), bkg_df[ic.selected_features].values
-    )
-    if bkg_predictions.ndim == 2:
-        bkg_predictions = bkg_predictions[:, multi_class_cut_branch]
+    bkg_predictions = bkg_df[["y_pred"]].values
+    bkg_predictions = bkg_predictions[:, multi_class_cut_branch]
     # prepare thresholds
     bin_array = np.array(range(-1000, 1000))
     thresholds = 1.0 / (1.0 + 1.0 / np.exp(bin_array * 0.02))
@@ -126,11 +130,16 @@ def get_significances(
             bkg_above_threshold.append(total_bkg_weights_passed)
     total_sig_weight = np.sum(sig_weights)
     total_bkg_weight = np.sum(bkg_weights)
-    return (plot_thresholds, significances, sig_above_threshold, bkg_above_threshold)
+    return (
+        plot_thresholds,
+        significances,
+        sig_above_threshold,
+        bkg_above_threshold,
+    )
 
 
 def plot_significance_scan(
-    model_wrapper, df, job_config, save_dir: pathlib.Path
+    df: pd.DataFrame, job_config: ht.config, save_dir: ht.pathlike
 ) -> None:
     """Shows significance change with threshold.
 
@@ -145,9 +154,7 @@ def plot_significance_scan(
         significances,
         sig_above_threshold,
         bkg_above_threshold,
-    ) = get_significances(
-        model_wrapper, df, job_config, significance_algo=significance_algo
-    )
+    ) = get_significances(df, job_config, significance_algo=significance_algo)
 
     significances_no_nan = np.nan_to_num(significances)
     max_significance = np.amax(significances_no_nan)
@@ -170,30 +177,45 @@ def plot_significance_scan(
     ax.set_title("significance scan")
     ax.axhline(y=original_significance, color="grey", linestyle="--")
     # significance scan curve
-    ax.plot(plot_thresholds, significances_no_nan, color="r", label=significance_algo)
+    ax.plot(
+        plot_thresholds,
+        significances_no_nan,
+        color="r",
+        label=significance_algo,
+    )
     # signal/background events scan curve
     ax2 = ax.twinx()
     max_sig_events = sig_above_threshold[0]
     max_bkg_events = bkg_above_threshold[0]
     sig_eff_above_threshold = np.array(sig_above_threshold) / max_sig_events
     bkg_eff_above_threshold = np.array(bkg_above_threshold) / max_bkg_events
-    ax2.plot(plot_thresholds, sig_eff_above_threshold, color="orange", label="sig")
-    ax2.plot(plot_thresholds, bkg_eff_above_threshold, color="blue", label="bkg")
+    ax2.plot(
+        plot_thresholds, sig_eff_above_threshold, color="orange", label="sig"
+    )
+    ax2.plot(
+        plot_thresholds, bkg_eff_above_threshold, color="blue", label="bkg"
+    )
     ax2.set_ylabel("sig(bkg) ratio after cut")
     # reference threshold
     ax.axvline(x=max_significance_threshold, color="green", linestyle="-.")
     # more infomation
     content = (
         "best threshold:"
-        + str(common_utils.get_significant_digits(max_significance_threshold, 6))
+        + str(
+            common_utils.get_significant_digits(max_significance_threshold, 6)
+        )
         + "\nmax significance:"
         + str(common_utils.get_significant_digits(max_significance, 6))
         + "\nbase significance:"
         + str(common_utils.get_significant_digits(original_significance, 6))
         + "\nsig events above threshold:"
-        + str(common_utils.get_significant_digits(max_significance_sig_total, 6))
+        + str(
+            common_utils.get_significant_digits(max_significance_sig_total, 6)
+        )
         + "\nbkg events above threshold:"
-        + str(common_utils.get_significant_digits(max_significance_bkg_total, 6))
+        + str(
+            common_utils.get_significant_digits(max_significance_bkg_total, 6)
+        )
     )
     ax.text(
         0.05,
@@ -232,9 +254,10 @@ def plot_significance_scan(
     fig.savefig(fig_save_path)
 
     # collect meta data
-    model_wrapper.original_significance = original_significance
-    model_wrapper.max_significance = max_significance
-    model_wrapper.max_significance_threshold = max_significance_threshold
+    # model_wrapper.original_significance = original_significance
+    # model_wrapper.max_significance = max_significance
+    # model_wrapper.max_significance_threshold = max_significance_threshold
+
     # make extra cut table 0.1, 0.2 ... 0.8, 0.9
     # make table for different DNN cut scores
     save_path = save_dir / "scan_DNN_cut.csv"
@@ -252,13 +275,22 @@ def plot_significance_scan(
         ]
         for index in range(1, 100):
             dnn_cut = (100 - index) / 100.0
-            threshold_id = (np.abs(np.array(plot_thresholds) - dnn_cut)).argmin()
+            threshold_id = (
+                np.abs(np.array(plot_thresholds) - dnn_cut)
+            ).argmin()
             sig_events = sig_above_threshold[threshold_id]
             sig_eff = sig_eff_above_threshold[threshold_id]
             bkg_events = bkg_above_threshold[threshold_id]
             bkg_eff = bkg_eff_above_threshold[threshold_id]
             significance = significances[threshold_id]
-            new_row = [dnn_cut, sig_events, sig_eff, bkg_events, bkg_eff, significance]
+            new_row = [
+                dnn_cut,
+                sig_events,
+                sig_eff,
+                bkg_events,
+                bkg_eff,
+                significance,
+            ]
             row_list.append(new_row)
         row_list.append([""])
         row_list.append(
@@ -297,7 +329,14 @@ def plot_significance_scan(
             bkg_events = bkg_above_threshold[threshold_id]
             bkg_eff = bkg_eff_above_threshold[threshold_id]
             significance = significances[threshold_id]
-            new_row = [dnn_cut, sig_events, sig_eff, bkg_events, bkg_eff, significance]
+            new_row = [
+                dnn_cut,
+                sig_events,
+                sig_eff,
+                bkg_events,
+                bkg_eff,
+                significance,
+            ]
             row_list.append(new_row)
         row_list.append([""])
         row_list.append(
@@ -336,7 +375,14 @@ def plot_significance_scan(
             bkg_events = bkg_above_threshold[threshold_id]
             bkg_eff = bkg_eff_cut
             significance = significances[threshold_id]
-            new_row = [dnn_cut, sig_events, sig_eff, bkg_events, bkg_eff, significance]
+            new_row = [
+                dnn_cut,
+                sig_events,
+                sig_eff,
+                bkg_events,
+                bkg_eff,
+                significance,
+            ]
             row_list.append(new_row)
         row_list.append([""])
         row_list.append(
